from bubot_helpers.ExtException import ExtException


class TelegramBotUpdate:
    def __init__(self, data):
        self.data = data
        self.bot_command = None
        self.bot_command_name = None
        self.bot_command_params = None

    @property
    def message(self):
        try:
            return self.data['message']
        except (AttributeError, TypeError, KeyError):
            pass
        try:
            return self.data['callback_query']['message']['reply_to_message']
        except (AttributeError, TypeError, KeyError):
            pass
        try:
            return self.data['callback_query']['message']
        except (AttributeError, TypeError, KeyError):
            pass
        return None

    @property
    def update_id(self):
        return self.data['update_id']

    @property
    def chat_id(self):
        chat = self.get_chat()
        if chat:
            return chat['id']
        return None

    @property
    def language(self):
        try:
            return self.message['from']['language_code']
        except (AttributeError, TypeError, KeyError):
            return None

    @property
    def message_id(self):
        try:
            return self.message['message_id']
        except (AttributeError, TypeError, KeyError):
            return None

    @property
    def callback_query(self):
        return self.data.get('callback_query')

    def is_user_contact(self):
        message = self.get_message()
        try:
            contact = message.get('contact')
        except (AttributeError, TypeError, KeyError):
            return False
        return contact and contact['user_id'] == message['from']['id']

    def is_bot_command(self):
        if self.bot_command is None:
            self.bot_command = False
            try:
                entities = self.message['entities']
            except (AttributeError, TypeError, KeyError):
                return None
            if entities[0].get('type') == 'bot_command':
                self.bot_command = True
                len_command_name = entities[0]['length']
                self.bot_command_name = self.message['text'][entities[0]['offset'] + 1: len_command_name]
                if len(self.message['text']) > len_command_name:
                    self.bot_command_params = self.message['text'][len_command_name + 1:]
        return self.bot_command

    def get_message(self):
        return self.message

    def get_chat(self):
        try:
            return self.message['chat']
        except (AttributeError, TypeError, KeyError):
            pass
        try:
            return self.data['my_chat_member']['chat']
        except (AttributeError, TypeError, KeyError):
            pass
        return None

    def is_change_member_status(self):
        if 'my_chat_member' not in self.data:
            return None
        event = self.data['my_chat_member']
        event['old_status'] = event['old_chat_member']['status']
        event['new_status'] = event['new_chat_member']['status']



import os.path

from bubot.core.BubotHelper import BubotHelper
from bubot_helpers.ExtException import ExtException, NotAvailable
from bubot.buject.OcfDevice.subtype.Device.Device import Device
from bubot.core.DataBase.Mongo import Mongo as Storage
from bubot.buject.OcfDevice.subtype.Device.RedisQueueMixin import RedisQueueMixin


class MessengerBotUpdater(RedisQueueMixin, Device):
    file = __file__
    template = False

    def __init__(self, **kwargs):
        self.storage = None
        Device.__init__(self, **kwargs)
        RedisQueueMixin.__init__(self, **kwargs)
        self.bot = None

    async def on_update_oic_con(self, message):
        pass

    async def on_pending(self):
        try:
            self.storage = await Storage.connect(self)
            await RedisQueueMixin.on_pending(self)
            bot_data = self.get_param('/oic/con', 'bot')
            handler = BubotHelper.get_subtype_class('MessengerBot', bot_data['subtype'])
            self.bot = handler(self.storage, device=self)
            self.bot = self.bot.init_by_data(bot_data)
            await self.bot.initialize()
            await Device.on_pending(self)
        except Exception as err:
            raise ExtException(parent=err)

    async def on_cancelled(self):
        await RedisQueueMixin.on_cancelled(self)
        await Device.on_cancelled(self)

    async def on_idle(self):
        try:
            await self.bot.scheduler()
        except Exception as err:
            await self._process_error(ExtException(parent=err, action='bot.scheduler'))
        try:
            try:
                limit = 100
                timeout = self.get_param('/oic/con', 'BotUpdateTimeout', 60)
                while True:
                    res = await self.bot.get_updates(timeout=timeout, limit=limit)
                    if len(res) < limit:
                        break
            except NotAvailable:
                pass
            finally:
                await self.bot.update()
        except Exception as err:
            await self._process_error(ExtException(parent=err, action='bot.update'))

    async def _process_error(self, err):
        msg = f'MessengerBotUpdater on_idle error: {str(err)}'
        self.log.error(msg)
        try:
            await self.bot.send_message_to_admin(msg)
        except Exception as err:
            self.log.error(f'send to admin {err}, {msg}')
            pass



import json
from asyncio import TimeoutError as AsyncioTimeoutError

import aiohttp

from bubot_helpers.ExtException import ExtException, NotAvailable, ExtTimeoutError
from bubot_helpers.Helper import Helper


class TelegramRawApi:
    api_url = 'https://api.telegram.org'

    def __init__(self, token, *, api_url=None):
        self._token = token
        self._bot_url = f'{api_url if api_url else self.api_url}/bot{token}'

    async def get_me(self):
        try:
            resp = await self.exec_method('getMe', request_type='get', timeout=20)
            resp['title'] = resp['first_name']
            return resp
        except Exception as err:
            raise ExtException(parent=err, action=f'{self.__class__.__name__}.get_me')

    async def get_updates(self, *, offset=None, limit=100, timeout=0, allowed_updates=None):
        try:
            data = dict(offset=offset, limit=limit, timeout=timeout, allowed_updates=allowed_updates)
            request_timeout = round(timeout * 1.3) if timeout else None
            resp = await self.exec_method('getUpdates', request_type='post', json=data, timeout=request_timeout)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action=f'{self.__class__.__name__}.get_updates')

    async def delete_message(self, chat_id: int, message_id: int):
        data = dict(
            chat_id=chat_id,
            message_id=message_id,
        )
        try:
            resp = await self.exec_method('deleteMessage', json=data)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action="delete_message", dump=data)

    async def send_message(self, chat_id: int, text: str, *, reply_markup=None, reply_to_message_id=None,
                           parse_mode=None):
        # print(f'send_message({chat_id}{text}')
        data = {
            'chat_id': chat_id,
            'text': text
        }
        Helper.obj_set_path_value(data, 'reply_markup', reply_markup, skip_if_none=True)
        Helper.obj_set_path_value(data, 'reply_to_message_id', reply_to_message_id, skip_if_none=True)
        Helper.obj_set_path_value(data, 'parse_mode', parse_mode, skip_if_none=True)
        try:
            resp = await self.exec_method('sendMessage', json=data)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action="send_message", dump=data)

    async def forward_message(self, chat_id: int, from_chat_id: int, message_id: int):
        data = {
            'chat_id': chat_id,
            'from_chat_id': from_chat_id,
            'message_id': message_id
        }
        try:
            resp = await self.exec_method('forwardMessage', json=data)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action="forward_message", dump=data)

    async def edit_message_text(self, chat_id: int, message_id: int, text: str, *, reply_markup=None,
                                parse_mode=None):
        # print(f'edit_message_text({chat_id}{text}')

        data = dict(
            chat_id=chat_id,
            message_id=message_id,
            text=text,
        )
        Helper.obj_set_path_value(data, 'reply_markup', reply_markup, skip_if_none=True)
        Helper.obj_set_path_value(data, 'parse_mode', parse_mode, skip_if_none=True)
        try:
            resp = await self.exec_method('editMessageText', json=data)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action="edit_message_text", dump=data)

    async def exec_method(self, method, *, request_type='post', json=None, timeout=None):
        try:
            url = f'{self._bot_url}/{method}'
            async with aiohttp.ClientSession() as session:
                async with session.request(request_type, url, json=json, timeout=timeout) as resp:
                    content_type = resp.headers.get('Content-Type')
                    if content_type.find('application/json') >= 0:
                        resp_data = await resp.json()

                        if resp.status in [200] and resp_data['ok']:
                            return resp_data['result']
                    if resp.status == 502:
                        raise ExtTimeoutError(message=await resp.text(), action="send_message")
                    raise NotImplementedError(f'{resp.status} {await resp.text()}')
        except AsyncioTimeoutError as err:
            raise NotAvailable(parent=err, action="exec_method")
        except aiohttp.ClientConnectionError as err:
            raise NotAvailable(parent=err, action="exec_method")
        except Exception as err:
            raise ExtException(parent=err, action='exec_method') from err

    @staticmethod
    def validate_web_app_data(init_data, bot_token):
        import hashlib
        import hmac

        from urllib.parse import unquote

        data_check_arr = unquote(init_data).split('&')
        data = {}
        hash_item = None
        for item in data_check_arr:
            key, value = item.split('=')
            data[key] = value
            if key == 'hash':
                hash_item = item
            elif key == 'user':
                data[key] = json.loads(value)
        data_check_arr.remove(hash_item)
        data_check_arr.sort()
        data_check_string = "\n".join(data_check_arr)

        secret_key = hmac.new('WebAppData'.encode(), bot_token.encode(), hashlib.sha256).digest()
        calculated_hash = hmac.new(secret_key, data_check_string.encode(), hashlib.sha256).hexdigest()
        if data['hash'] != calculated_hash:
            return None
        return data



from bubot_helpers.ActionDecorator import async_action
from bubot.buject.User.User import User
from bubot.core.ObjApi import ObjApi
from bubot_helpers.ActionDecorator import async_action
from bubot_helpers.ExtException import ExtException

from bubot_messenger_bot.buject.MessengerBot.Helper import clear_phone
from bubot_messenger_bot.buject.MessengerBot.TelegramBot import TelegramBot as Bot
from bubot_messenger_bot.buject.MessengerBotChat.MessengerBotChat import MessengerBotChat as Chat
from bubot.buject.Session.Session import Session

# from bubot_messenger_bot.buject.Client.Client import Client
# from bubot_messenger_bot.buject.Order.Order import Order


class MessengerBotApi(ObjApi):
    # client = Client
    # handler: Order = Order

    bot_data = {}

    @async_action
    async def public_api_telegram_webhook(self, view, *, _action=None, **kwargs):
        data = await view.loads_json_request_data(view)
        pass

    @staticmethod
    def get_bot_instance(view, bot_id):
        # todo убрать говнокод
        running_devices = view.app['device'].running_devices
        for uuid in running_devices:
            if running_devices[uuid].__class__.__name__ == 'MessengerBotUpdater':
                if running_devices[uuid].bot.data['token'].startswith(str(bot_id)):
                    return running_devices[uuid].bot
        raise NotImplementedError()

    @async_action
    async def public_api_sign_in_by_web_app(self, view, *, _action=None, **kwargs):
        data = await view.loads_json_request_data(view)
        # todo убрать говнокод
        bot_id = int(data['bot'])
        log = view.app['device'].log
        log.info(data)

        try:
            bot_data = self.bot_data[bot_id]
        except KeyError:
            bot = Bot(view.storage)
            await bot.find_by_id(bot_id, _form=None)
            bot_data = bot.data
            self.bot_data[bot_id] = bot_data
        token = bot_data['token']

        init_data = Bot.raw_api.validate_web_app_data(data['initData'], token)
        if init_data is None:
            log.error(data)
            raise ExtException(message='Bad initData', detail='Что ты вы делаете не то, обратитесь к разработчику')
        user = User(view.storage, lang=view.lang, form='CurrentUser')

        user_id = init_data['user']['id']
        title = f"{init_data['user']['last_name']} {init_data['user']['first_name']}"

        chat = Chat(view.storage, bot_id=int(bot_id))
        await chat.find_by_chat_id(int(user_id))

        phone = clear_phone(chat.data.get('phone'))
        if not phone:
            raise ExtException(message='Not phone')

        auth_type = 'phone'
        try:
            _action.add_stat(await user.add_auth({
                'type': auth_type,
                'id': phone,
                'title': title
            }, **kwargs))
        except ExtException as err:
            pass

        _action.add_stat(await Session.create_from_request(user, view,
                                                           bot_id=bot_id,
                                                           user_id=user_id
                                                           ))
        return self.response.json_response({
            'title': chat.data['title'],
            'bot': bot_data['subtype'],
            'bot_id': int(bot_id),
            'chat_id': int(user_id),
            'phone': phone,
            # 'login': chat.data.get('login'),
            # 'password': chat.data.get('password'),
            # 'access': chat.data.get('access'),
        })



import json
from asyncio import TimeoutError as AsyncioTimeoutError
import hmac
import hashlib

import aiohttp
from bubot_helpers.ExtException import ExtException, NotAvailable, ExtTimeoutError


class ViberRawApi:
    api_url = 'https://chatapi.viber.com/pa/'

    def __init__(self, token, *, api_url=None):
        self._token = token
        if api_url:
            self.api_url = api_url

    async def get_me(self):
        try:
            resp = await self.exec_method('get_account_info', request_type='get', timeout=20)
            resp['title'] = resp['name']
            return resp
        except Exception as err:
            raise ExtException(parent=err, action=f'{self.__class__.__name__}.get_me')

    async def set_webhook(self, url):
        if not url:
            raise ExtException(message='Webhook url not defined')
        data = dict(
            url=url,
            event_types=[
                "failed",
                "subscribed",
                "unsubscribed",
                "conversation_started"
            ],
            send_name=True,
            send_photo=False
        )
        resp = await self.exec_method('set_webhook', json=data)
        a = 1

    def verify_signature(self, request_data, signature):
        return signature == self._calculate_message_signature(request_data)

    def _calculate_message_signature(self, message):
        return hmac.new(
            bytes(self._token.encode('ascii')),
            msg=message,
            digestmod=hashlib.sha256) \
            .hexdigest()

    # async def get_updates(self, *, offset=None, limit=100, timeout=0, allowed_updates=None):
    #     try:
    #         data = dict(offset=offset, limit=limit, timeout=timeout, allowed_updates=allowed_updates)
    #         request_timeout = round(timeout * 1.3) if timeout else None
    #         resp = await self.exec_method('getUpdates', request_type='post', json=data, timeout=request_timeout)
    #         return resp
    #     except Exception as err:
    #         raise ExtException(parent=err, action=f'{self.__class__.__name__}.get_updates')
    #
    # async def delete_message(self, chat_id: int, message_id: int):
    #     data = dict(
    #         chat_id=chat_id,
    #         message_id=message_id,
    #     )
    #     try:
    #         resp = await self.exec_method('deleteMessage', json=data)
    #         return resp
    #     except Exception as err:
    #         raise ExtException(parent=err, action="delete_message", dump=data)
    #
    async def send_message(self, chat_id: int, data):
        try:
            resp = await self.exec_method('send_message', json=data)
            return resp
        except Exception as err:
            raise ExtException(parent=err, action="send_message", dump=data)
    #
    # async def edit_message_text(self, chat_id: int, message_id: int, text: str, *, reply_markup=None,
    #                             parse_mode=None):
    #     # print(f'edit_message_text({chat_id}{text}')
    #
    #     data = dict(
    #         chat_id=chat_id,
    #         message_id=message_id,
    #         text=text,
    #     )
    #     Helper.obj_set_path_value(data, 'reply_markup', reply_markup, skip_if_none=True)
    #     Helper.obj_set_path_value(data, 'parse_mode', parse_mode, skip_if_none=True)
    #     try:
    #         resp = await self.exec_method('editMessageText', json=data)
    #         return resp
    #     except Exception as err:
    #         raise ExtException(parent=err, action="edit_message_text", dump=data)
    #
    async def exec_method(self, method, *, request_type='post', json=None, timeout=None):
        try:
            url = f'{self.api_url}/{method}'
            headers = {'X-Viber-Auth-Token': self._token}
            async with aiohttp.ClientSession() as session:
                async with session.request(request_type, url, headers=headers, json=json, timeout=timeout) as resp:
                    content_type = resp.headers.get('Content-Type')
                    if content_type.find('application/json') >= 0:
                        resp_data = await resp.json()

                        if resp.status in [200]:
                            if resp_data['status'] == 0:
                                return resp_data
                            else:
                                raise ExtException(message=resp_data['status_message'])

                    if resp.status == 502:
                        raise ExtTimeoutError(message=await resp.text(), action="send_message")
                    raise NotImplementedError(f'{resp.status} {await resp.text()}')
        except AsyncioTimeoutError as err:
            raise NotAvailable(parent=err, action="exec_method")
        except aiohttp.ClientConnectionError as err:
            raise NotAvailable(parent=err, action="exec_method")
        except Exception as err:
            raise ExtException(parent=err, action='exec_method') from err

    @staticmethod
    def validate_web_app_data(init_data, bot_token):
        import hashlib
        import hmac

        from urllib.parse import unquote

        data_check_arr = unquote(init_data).split('&')
        data = {}
        hash_item = None
        for item in data_check_arr:
            key, value = item.split('=')
            data[key] = value
            if key == 'hash':
                hash_item = item
            elif key == 'user':
                data[key] = json.loads(value)
        data_check_arr.remove(hash_item)
        data_check_arr.sort()
        data_check_string = "\n".join(data_check_arr)

        secret_key = hmac.new('WebAppData'.encode(), bot_token.encode(), hashlib.sha256).digest()
        calculated_hash = hmac.new(secret_key, data_check_string.encode(), hashlib.sha256).hexdigest()
        if data['hash'] != calculated_hash:
            return None
        return data



import json
from datetime import datetime

from bubot_helpers.ArrayHelper import ArrayHelper
from bubot_helpers.ExtException import KeyNotFound, ExtException
from bubot_helpers.Helper import get_tzinfo
from bubot_messenger_bot.buject.MessengerBot.MessengerBot import MessengerBot
from bubot_messenger_bot.buject.MessengerBot.Helper import clear_phone
from bubot_messenger_bot.buject.MessengerBot.TelegramBotUpdate import TelegramBotUpdate as BotUpdate
from bubot_messenger_bot.buject.MessengerBot.TelegramRawApi import TelegramRawApi as Api
from bubot_messenger_bot.buject.MessengerBotChat.MessengerBotChat import MessengerBotChat as Chat

tz_info = get_tzinfo()


class TelegramBot(MessengerBot):
    file = __file__  # должен быть в каждом файле наследнике для чтения форм
    selenium_scenario = None
    raw_api = Api

    async def get_updates(self, *, timeout=0, limit=100):
        last_update_id = self.data.get('last_update_id', 0)
        if last_update_id:
            last_update_id += 1
        updates = await self.api.get_updates(offset=last_update_id, timeout=timeout, limit=limit)
        try:
            for _update in updates:
                update = BotUpdate(_update)
                await self.process_update(update)
                self.data['last_update_id'] = update.update_id
        except Exception as err:
            raise ExtException(parent=err, action='TelegramBot.get_updates') from err
        return updates

    async def process_update(self, update: BotUpdate):
        try:
            if update.chat_id:
                if update.chat_id < 0:
                    # resp = await self.api.send_message(
                    #     update.chat_id,
                    #     'Бот не работает в группах.'
                    # )
                    return
                chat = Chat(self.storage, bot_id=self.obj_id)
                try:
                    await chat.find_by_chat_id(update.chat_id)
                    if not chat.lang:
                        chat.lang = update.language
                except KeyNotFound:
                    chat_data = update.get_chat()
                    title = f"{chat_data.get('last_name', '')} {chat_data.get('first_name', '')}".strip()
                    chat.init_by_data({
                        'bot_id': self.obj_id,
                        'chat_id': update.chat_id,
                        'title': title,
                        'username': chat_data.get('username'),
                        'lang': update.language
                    })
                    await self.send_message_to_admin(f'New chat {self.__class__.__name__} {update.chat_id} {title}')

                await self.process_chat_update(update, chat=chat)
                await chat.update()
            else:
                await self.send_message_to_admin(f'Unsupported update {json.dumps(update.data, ensure_ascii=False)}')
        except Exception as err:
            raise ExtException(parent=err, dump=update.data)

    async def process_chat_update(self, update: BotUpdate, *, chat=None):
        try:
            if update.callback_query:
                try:
                    command = update.message['text'][1:]
                    await getattr(self, f'process_callback_{command}')(update, chat=chat)
                except Exception as err:
                    self.log.error(err)
            elif update.is_bot_command():
                try:
                    await getattr(self, f'process_command_{update.bot_command_name}')(update, chat=chat)
                except Exception as err:
                    self.log.error(err)
            elif update.is_user_contact():
                await self.process_command_set_phone(update, chat=chat)
            else:
                if chat:
                    my_chat_member = update.data.get('my_chat_member')
                    if my_chat_member:
                        new_chat_member = my_chat_member.get('new_chat_member')
                        if new_chat_member.get('status') == 'member':
                            await getattr(self, 'process_command_start')(update, chat=chat)
                        return

                    next_update = chat.data.get('next_update')
                    if next_update:
                        if next_update['type'] == 'command_param':
                            update.bot_command_name = next_update['command']
                            update.bot_command_params = update.get_message().get('text', '')
                            command_handler = f'process_command_{update.bot_command_name}'
                            try:
                                await getattr(self, command_handler)(update, chat=chat)
                            except Exception as err:
                                err = ExtException(parent=err, action=command_handler)
                                self.log.error(err)
                        chat.data['next_update'] = None
                    else:
                        if update.message_id:
                            await self.process_message(chat, update)
                        else:
                            await self.send_message_to_admin(
                                f'Unsupported update {json.dumps(update.data, ensure_ascii=False)}')
                else:
                    await self.send_message_to_admin(
                        f'Unsupported update {json.dumps(update.data, ensure_ascii=False)}')
                pass
        except Exception as err:
            err1 = ExtException(parent=err, dump=update.data)
            chat.data['last_error'] = err1.to_dict()
            await self.send_message_to_admin(
                f'Unsupported update {err1} {json.dumps(update.data, ensure_ascii=False)}')

    async def process_message(self, chat, update):
        try:
            if 'messages' not in chat.data:
                chat.data['messages'] = []
                index = -1
            else:
                index = ArrayHelper.find_by_key(chat.data['messages'], update.message_id, '_id')
            # _raw_json = json.dumps(update.data, ensure_ascii=False)
            _message = {
                'text': update.data['message'].get('text'),
                'date': datetime.fromtimestamp(update.data['message']['date'], tz=tz_info),
                '_id': update.data['message']['message_id']
            }
            # _message.pop('chat', None)
            # _message.pop('from', None)
            # _message['_id'] = _message.pop('message_id')
            # _message['__update_id'] = update.update_id
            # _message['date'] = datetime.fromtimestamp(_message['date'], tz=tz_info)
            if 'reply_to_message' in update.message and update.message['reply_to_message']['from']['id'] == self.data['_id']:
                # пользователь запретил пересылку настройками конфиденциальности
                original_chat = Chat(self.storage, bot_id=self.obj_id)
                forward_origin =  update.message['reply_to_message']['forward_origin']
                try:
                    if 'forward_from' not in update.message['reply_to_message']:
                        original_chat_id = None
                        user_name = forward_origin['sender_user_name']
                        message_date = datetime.fromtimestamp(forward_origin['date'], tz=tz_info)
                        await original_chat.find_by_message_date_and_user_name(message_date, user_name)
                    else:
                        original_chat_id = update.message['reply_to_message']['forward_from']['id']
                        await original_chat.find_by_chat_id(original_chat_id)
                    res = await self.api.send_message(original_chat.chat_id, _message['text'])
                    _message['_id'] = res['message_id']
                    _message['bot'] = True
                    original_chat.data['messages'].append(_message)
                    await original_chat.update()
                    return
                except KeyNotFound:
                    await self.send_message_to_admin(
                            f"Сообщение не доставлено до получателя "
                            f"{forward_origin['sender_user_name']}")
                    return
            if index < 0:
                if _message['text']:
                    chat.data['messages'].append(_message)
                await self.forward_message_to_admin(update)
                # await self.send_message_to_admin(f'Message {_raw_json}', from_chat=chat.chat_id)
            else:
                chat.data['messages'][index] = _message
        except Exception as err:
            raise ExtException(parent=err) from err

    async def get_list_admin_chat(self):
        filter = {
            "bot_id": self.obj_id,
            "admin": True
        }
        chat = Chat(self.storage, bot_id=self.obj_id)
        chats = await chat.list(filter=filter)
        return chats.result['Rows']

    async def send_message_to_admin(self, text, *, reply_to_message_id=None, parse_mode=None, from_chat=None):
        self.log.info(f'send_message_to_admin: {text}')
        admin_chats = await self.get_list_admin_chat()
        if not admin_chats:
            self.log.error(f'Admin chat not found: bot{self.title}')
            return
        for admin_data in admin_chats:
            if from_chat and from_chat == admin_data['chat_id']:
                continue
            await self.api.send_message(admin_data['chat_id'], text, reply_to_message_id=reply_to_message_id,
                                        parse_mode=parse_mode)

    async def forward_message_to_admin(self, update):
        try:
            # self.log.error(f'forward_message_to_admin: {text}')
            from_chat_id = update.message['chat']['id']
            message_id = update.message_id
            admin_chats = await self.get_list_admin_chat()
            for admin_data in admin_chats:
                if from_chat_id and from_chat_id == admin_data['chat_id']:
                    continue
                await self.api.forward_message(admin_data['chat_id'], from_chat_id, message_id)
        except Exception as err:
            raise ExtException(parent=err)

    async def process_command_start(self, update: BotUpdate, *, chat: Chat = None):
        await self.process_command_help(update, chat=chat)

    async def process_command_help(self, update: BotUpdate, *, chat: Chat = None):
        await self.api.send_message(update.chat_id, self.t('start', update.language),
                                    reply_to_message_id=update.message_id,
                                    parse_mode='Markdown')

    async def process_command_set_phone(self, update: BotUpdate, *, chat: Chat = None):
        message = update.get_message()
        contact = message.get('contact')
        if contact and contact['user_id'] == message['from']['id']:
            chat.data['phone'] = clear_phone(contact['phone_number'])

    async def scheduler(self):
        pass



import json
from datetime import datetime

from bubot_helpers.ArrayHelper import ArrayHelper
from bubot_helpers.ExtException import KeyNotFound, ExtException
from bubot_helpers.Helper import get_tzinfo

from bubot_messenger_bot.buject.MessengerBot.MessengerBot import MessengerBot
from bubot_messenger_bot.buject.MessengerBot.ViberBotUpdate import ViberBotUpdate as BotUpdate
from bubot_messenger_bot.buject.MessengerBot.ViberRawApi import ViberRawApi as Api
from bubot_messenger_bot.buject.MessengerBotChat.MessengerBotChat import MessengerBotChat as Chat

tz_info = get_tzinfo()


class ViberBot(MessengerBot):
    file = __file__  # должен быть в каждом файле наследнике для чтения форм
    raw_api = Api

    async def initialize(self):
        await super().initialize()
        await self.api.set_webhook(self.get_webhook_url())

    def get_webhook_url(self):
        raise NotImplementedError()

    async def on_webhook(self, data):
        return {}

    async def on_message(self, data):

        return {}

    async def get_updates(self, *, timeout=0, limit=100):
        return []

    async def scheduler(self):
        pass



from datetime import datetime

from bubot.core.ObjApi import ObjApi
from bubot_helpers.ActionDecorator import async_action
from bubot_messenger_bot.buject.MessengerBot.MessengerBotApi import MessengerBotApi
from .MessengerBotChat import MessengerBotChat


class MessengerBotChatApi(ObjApi):
    handler = MessengerBotChat

    @async_action
    async def api_read(self, view, *, _action=None, **kwargs):
        handler, data = await self.prepare_json_request(view)
        await self.check_right(view, handler, 11)
        result = _action.add_stat(await handler.find_one(data, _form=None))
        return self.response.json_response(result)

    @async_action
    async def api_send_message(self, view, *, _action=None, **kwargs):
        handler, data = await self.prepare_json_request(view)
        await self.check_right(view, handler, 11)
        await self.send_message(view, handler, data, _action=_action)
        return self.response.json_response(handler.data)

    async def send_message(self, view, handler, data, *, _action=None):
        bot_id = data['bot_id']
        chat_id = data['chat_id']
        message = data['message']
        _action.add_stat(await handler.find_one(dict(
            bot_id=bot_id,
            chat_id=chat_id
        ), _form=None))

        bot = MessengerBotApi.get_bot_instance(view, bot_id)
        res = await bot.api.send_message(
            chat_id,
            message
        )
        if 'messages' not in handler.data:
            handler.data['messages'] = []

        handler.data['messages'].append(dict(
            date=datetime.fromtimestamp(res['date']),
            text=message,
            bot=True
        ))
        _action.add_stat(await handler.update())



import setuptools
from src.bubot_messenger_bot import __version__

# with open("README.md", "r", encoding="utf-8") as fh:
#     long_description = fh.read()

setuptools.setup(
    name='bubot_messenger_bot',
    version=__version__,
    author="Razgovorov Mikhail",
    author_email="1338833@gmail.com",
    description="",
    # long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/razgovorov/bubot_messenger_bot.git",
    package_dir={'': 'src'},
    package_data={
        '': ['*.md', '*.json'],
    },
    packages=setuptools.find_namespace_packages(where='src'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Framework :: AsyncIO",
    ],
    python_requires='>=3.8',
    zip_safe=False,
    install_requires=[
        'bubot_core>=4.1.0',
        'bubot_helpers>=4.0.3',
    ]
)



from bubot.core.Obj import Obj

from bubot_helpers.ActionDecorator import async_action


class MessengerBotChat(Obj):
    name = 'TelegramBotChat'
    file = __file__

    @property
    def db(self):
        return 'ToFirstGrade'

    def __init__(self, storage, *, account_id=None, lang=None, data=None, bot_id=None, **kwargs):
        super().__init__(storage, account_id=account_id, lang=lang, data=data, **kwargs)
        self.bot_id = bot_id

    def init(self):
        self.data = dict(
            bot_id=self.bot_id,
        )

    @property
    def chat_id(self):
        return self.data.get("chat_id")

    @property
    def bot_id(self):
        return self.data.get("bot_id")

    @bot_id.setter
    def bot_id(self, value):
        self.data['bot_id'] = value

    @property
    def lang(self):
        return self.data.get("lang")

    @lang.setter
    def lang(self, value):
        self.data['lang'] = value

    @async_action
    async def find_by_chat_id(self, chat_id, *, _form="Item", _action=None, **kwargs):
        return await self.find_one({"bot_id": self.bot_id, "chat_id": chat_id}, _form=_form, **kwargs)

    async def find_by_message_date_and_user_name(self, date, user_name, *, _form="Item", _action=None, **kwargs):
        return await self.find_one({"bot_id": self.bot_id, "title": user_name, "messages.date": date}, _form=_form, **kwargs)



import json
import logging
from datetime import datetime

from bubot.core.Obj import Obj
from bubot_helpers.ArrayHelper import ArrayHelper
from bubot_helpers.ExtException import KeyNotFound, ExtException
from bubot_helpers.Helper import get_tzinfo
# from bubot_messenger_bot.buject.MessengerBot.MessengerBotUpdate import BotUpdate
# from bubot_messenger_bot.buject.ViberBot.RawViberApi import RawViberApi as Api
# from bubot_messenger_bot.buject.ViberBotChat.ViberBotChat import ViberBotChat as Chat

tz_info = get_tzinfo()


class MessengerBot(Obj):
    file = __file__  # должен быть в каждом файле наследнике для чтения форм
    name = 'TelegramBot'
    raw_api = None

    def __init__(self, storage, *, device=None, account_id=None, lang=None, data=None, **kwargs):
        self.device = device
        self.storage = storage
        self.api = None
        self.log = logging.getLogger(f'{self.__class__.__name__}')
        super().__init__(self.storage, account_id=account_id, lang=lang, data=data, **kwargs)

    @property
    def db(self):
        return 'ToFirstGrade'

    def init(self):
        self.data = dict(
            subtype=self.__class__.__name__,
            title=self.__class__.__name__
        )

    def init_by_data(self, data):
        obj_class = super().init_by_data(data)
        obj_class.api = obj_class.raw_api(data['token'])
        return obj_class

    async def initialize(self):
        try:
            me = await self.api.get_me()
            self.obj_id = me['id']
            self.data['title'] = me['title']
            await self.update()
        except Exception as err:
            raise ExtException(parent=err)

